﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.Graphics.Imaging;
using Windows.Storage;
using Windows.Storage.Streams;
using Windows.UI.StartScreen;

namespace SecondaryTileSample
{
	public static class TileUtilities
	{
		public async static Task CreateSecondaryTileFromWebImage(
			string tileId, Uri imageUri, string shortName, string displayName,
			string arguments, Rect selection)
		{
			//Download image to LocalFolder and use the tileId as the identifier
			string filename = string.Format("{0}.png", tileId);
			HttpClient httpClient = new HttpClient();
			var response = await httpClient.GetAsync(imageUri);
			var imageFile = await ApplicationData.Current.LocalFolder.CreateFileAsync(filename, CreationCollisionOption.ReplaceExisting);
			using (var fs = await imageFile.OpenAsync(FileAccessMode.ReadWrite))
			{
				using (var outStream = fs.GetOutputStreamAt(0))
				{
					DataWriter writer = new DataWriter(outStream);
					writer.WriteBytes(await response.Content.ReadAsByteArrayAsync());
					await writer.StoreAsync();
					writer.DetachStream();
					await outStream.FlushAsync();
				}
			}
			await DarkenImageBottom(filename, filename);
			//Create tile
			Uri image = new Uri(string.Format("ms-appdata:///local/{0}", filename));
			SecondaryTile secondaryTile = new SecondaryTile(tileId, shortName, displayName, arguments, TileOptions.ShowNameOnLogo, image);
			secondaryTile.ForegroundText = ForegroundText.Light;
			await secondaryTile.RequestCreateForSelectionAsync(selection, Windows.UI.Popups.Placement.Above);
		}


		private async static Task DarkenImageBottom(string filename, string outfilename)
		{
			var file = await ApplicationData.Current.LocalFolder.GetFileAsync(filename);
			BitmapDecoder decoder = null;
			byte[] sourcePixels = null;
			using (IRandomAccessStream fileStream = await file.OpenReadAsync())
			{
				decoder = await BitmapDecoder.CreateAsync(fileStream);
				// Scale image to appropriate size 
				BitmapTransform transform = new BitmapTransform();
				PixelDataProvider pixelData = await decoder.GetPixelDataAsync(
					BitmapPixelFormat.Bgra8,
					BitmapAlphaMode.Straight,
					transform,
					ExifOrientationMode.IgnoreExifOrientation, // This sample ignores Exif orientation 
					ColorManagementMode.DoNotColorManage
				);
				// An array containing the decoded image data, which could be modified before being displayed 
				sourcePixels = pixelData.DetachPixelData();
				fileStream.Dispose();
			}
			if (decoder != null && sourcePixels != null)
			{
				for (uint col = 0; col < decoder.PixelWidth; col++)
				{
					for (uint row = (uint)(decoder.PixelHeight * .6); row < decoder.PixelHeight; row++)
					{
						uint idx = (row * decoder.PixelWidth + col) * 4;
						if (decoder.BitmapPixelFormat == BitmapPixelFormat.Bgra8 ||
							decoder.BitmapPixelFormat == BitmapPixelFormat.Rgba8)
						{
							var frac = 1 - Math.Sin(((row / (double)decoder.PixelHeight) - .6) * (1 / .4));
							byte b = sourcePixels[idx];
							byte g = sourcePixels[idx + 1];
							byte r = sourcePixels[idx + 2];
							sourcePixels[idx] = (byte)(b * frac);
							sourcePixels[idx + 1] = (byte)(g * frac);
							sourcePixels[idx + 2] = (byte)(r * frac);
						}
					}
				}

				var file2 = await ApplicationData.Current.LocalFolder.CreateFileAsync(outfilename, CreationCollisionOption.ReplaceExisting);

				var str = await file2.OpenAsync(Windows.Storage.FileAccessMode.ReadWrite);
				BitmapEncoder enc = await BitmapEncoder.CreateAsync(BitmapEncoder.PngEncoderId, str);
				enc.SetPixelData(BitmapPixelFormat.Bgra8, BitmapAlphaMode.Ignore, decoder.PixelWidth, decoder.PixelHeight,
					decoder.DpiX, decoder.DpiY, sourcePixels);
				await enc.FlushAsync();
				str.Dispose();
			}
		}
	}
}
